@@ -123,6 +123,7 @@ class Firewall(object):
         self._firewall_backend = config.FALLBACK_FIREWALL_BACKEND
         self._flush_all_on_reload = config.FALLBACK_FLUSH_ALL_ON_RELOAD
         self._rfc3964_ipv4 = config.FALLBACK_RFC3964_IPV4
+        self._allow_zone_drifting = config.FALLBACK_ALLOW_ZONE_DRIFTING
 
     def individual_calls(self):
         return self._individual_calls
@@ -286,6 +287,20 @@ class Firewall(object):
                 log.debug1("RFC3964_IPv4 is set to '%s'",
                            self._rfc3964_ipv4)
 
+            if self._firewalld_conf.get("AllowZoneDrifting"):
+                value = self._firewalld_conf.get("AllowZoneDrifting")
+                if value.lower() in [ "no", "false" ]:
+                    self._allow_zone_drifting = False
+                else:
+                    self._allow_zone_drifting = True
+                    if not self._offline:
+                        log.warning("AllowZoneDrifting is enabled. This is considered "
+                                    "an insecure configuration option. It will be "
+                                    "removed in a future release. Please consider "
+                                    "disabling it now.")
+                log.debug1("AllowZoneDrifting is set to '%s'",
+                           self._allow_zone_drifting)
+
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         self._select_firewall_backend(self._firewall_backend)
@@ -628,6 +643,7 @@ class Firewall(object):
     def stop(self):
         if self.cleanup_on_exit and not self._offline:
             self.flush()
+            self.ipset.flush()
             self.set_policy("ACCEPT")
             self.modules_backend.unload_firewall_modules()
 
@@ -932,6 +948,10 @@ class Firewall(object):
             _direct_config = self.direct.get_runtime_config()
             _old_dz = self.get_default_zone()
 
+        _ipset_objs = []
+        for _name in self.ipset.get_ipsets():
+            _ipset_objs.append(self.ipset.get_ipset(_name))
+
         if not _panic:
             self.set_policy("DROP")
 
@@ -946,6 +966,16 @@ class Firewall(object):
             # etc. We'll re-raise it at the end.
             start_exception = e
 
+        # destroy ipsets no longer in the permanent configuration
+        if flush_all:
+            for obj in _ipset_objs:
+                if not self.ipset.query_ipset(obj.name):
+                    for backend in self.ipset.backends():
+                        # nftables sets are part of the normal firewall ruleset.
+                        if backend.name == "nftables":
+                            continue
+                        backend.set_destroy(obj.name)
+
         if not flush_all:
             # handle interfaces in the default zone and move them to the new
             # default zone if it changed
@@ -980,6 +1010,19 @@ class Firewall(object):
                     del _zone_interfaces[zone]
             del _zone_interfaces
 
+            # restore runtime-only ipsets
+            for obj in _ipset_objs:
+                if self.ipset.query_ipset(obj.name):
+                    for entry in obj.entries:
+                        try:
+                            self.ipset.add_entry(obj.name, entry)
+                        except FirewallError as msg:
+                            if msg.code != errors.ALREADY_ENABLED:
+                                raise msg
+                else:
+                    self.ipset.add_ipset(obj)
+                    self.ipset.apply_ipset(obj.name)
+
             # restore direct config
             self.direct.set_config(_direct_config)
 
