@@ -41,7 +41,6 @@ from firewall.server.config_zone import FirewallDConfigZone
 from firewall.server.config_ipset import FirewallDConfigIPSet
 from firewall.server.config_helper import FirewallDConfigHelper
 from firewall.core.io.zone import Zone
-from firewall.core.io.service import Service
 from firewall.core.io.icmptype import IcmpType
 from firewall.core.io.ipset import IPSet
 from firewall.core.io.helper import Helper
@@ -106,6 +105,9 @@ class FirewallDConfig(slip.dbus.service.Object):
                                                 "LogDenied": "readwrite",
                                                 "AutomaticHelpers": "readwrite",
                                                 "FirewallBackend": "readwrite",
+                                                "FlushAllOnReload": "readwrite",
+                                                "RFC3964_IPv4": "readwrite",
+                                                "AllowZoneDrifting": "readwrite",
                                               })
 
     @handle_exceptions
@@ -485,7 +487,9 @@ class FirewallDConfig(slip.dbus.service.Object):
     def _get_property(self, prop):
         if prop not in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                          "Lockdown", "IPv6_rpfilter", "IndividualCalls",
-                         "LogDenied", "AutomaticHelpers", "FirewallBackend" ]:
+                         "LogDenied", "AutomaticHelpers", "FirewallBackend",
+                         "FlushAllOnReload", "RFC3964_IPv4",
+                         "AllowZoneDrifting" ]:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.InvalidArgs: "
                 "Property '%s' does not exist" % prop)
@@ -530,6 +534,18 @@ class FirewallDConfig(slip.dbus.service.Object):
             if value is None:
                 value = config.FALLBACK_FIREWALL_BACKEND
             return dbus.String(value)
+        elif prop == "FlushAllOnReload":
+            if value is None:
+                value = "yes" if config.FALLBACK_FLUSH_ALL_ON_RELOAD else "no"
+            return dbus.String(value)
+        elif prop == "RFC3964_IPv4":
+            if value is None:
+                value = "yes" if config.FALLBACK_RFC3964_IPV4 else "no"
+            return dbus.String(value)
+        elif prop == "AllowZoneDrifting":
+            if value is None:
+                value = "yes" if config.FALLBACK_ALLOW_ZONE_DRIFTING else "no"
+            return dbus.String(value)
 
     @dbus_handle_exceptions
     def _get_dbus_property(self, prop):
@@ -551,6 +567,12 @@ class FirewallDConfig(slip.dbus.service.Object):
             return dbus.String(self._get_property(prop))
         elif prop == "FirewallBackend":
             return dbus.String(self._get_property(prop))
+        elif prop == "FlushAllOnReload":
+            return dbus.String(self._get_property(prop))
+        elif prop == "RFC3964_IPv4":
+            return dbus.String(self._get_property(prop))
+        elif prop == "AllowZoneDrifting":
+            return dbus.String(self._get_property(prop))
         else:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.InvalidArgs: "
@@ -590,7 +612,9 @@ class FirewallDConfig(slip.dbus.service.Object):
         if interface_name == config.dbus.DBUS_INTERFACE_CONFIG:
             for x in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                        "Lockdown", "IPv6_rpfilter", "IndividualCalls",
-                       "LogDenied", "AutomaticHelpers", "FirewallBackend" ]:
+                       "LogDenied", "AutomaticHelpers", "FirewallBackend",
+                       "FlushAllOnReload", "RFC3964_IPv4",
+                       "AllowZoneDrifting" ]:
                 ret[x] = self._get_property(x)
         elif interface_name in [ config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                                  config.dbus.DBUS_INTERFACE_CONFIG_POLICIES ]:
@@ -617,7 +641,8 @@ class FirewallDConfig(slip.dbus.service.Object):
             if property_name in [ "MinimalMark", "CleanupOnExit", "Lockdown",
                                   "IPv6_rpfilter", "IndividualCalls",
                                   "LogDenied", "AutomaticHelpers",
-                                  "FirewallBackend" ]:
+                                  "FirewallBackend", "FlushAllOnReload",
+                                  "RFC3964_IPv4", "AllowZoneDrifting" ]:
                 if property_name == "MinimalMark":
                     try:
                         int(new_value)
@@ -651,6 +676,22 @@ class FirewallDConfig(slip.dbus.service.Object):
                         raise FirewallError(errors.INVALID_VALUE,
                                             "'%s' for %s" % \
                                             (new_value, property_name))
+                if property_name == "FlushAllOnReload":
+                    if new_value.lower() not in ["yes", "true", "no", "false"]:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
+                if property_name == "RFC3964_IPv4":
+                    if new_value.lower() not in ["yes", "true", "no", "false"]:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
+                if property_name == "AllowZoneDrifting":
+                    if new_value.lower() not in ["yes", "true", "no", "false"]:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
+
                 self.config.get_firewalld_conf().set(property_name, new_value)
                 self.config.get_firewalld_conf().write()
                 self.PropertiesChanged(interface_name,
@@ -1037,7 +1078,7 @@ class FirewallDConfig(slip.dbus.service.Object):
         raise FirewallError(errors.INVALID_SERVICE, service)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
-                         in_signature='s'+Service.DBUS_SIGNATURE,
+                         in_signature='s(sssa(ss)asa{ss}asa(ss))',
                          out_signature='o')
     @dbus_handle_exceptions
     def addService(self, service, settings, sender=None):
@@ -1051,6 +1092,21 @@ class FirewallDConfig(slip.dbus.service.Object):
         config_service = self._addService(obj)
         return config_service
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
+                         in_signature='sa{sv}',
+                         out_signature='o')
+    @dbus_handle_exceptions
+    def addService2(self, service, settings, sender=None):
+        """add service with given name and settings
+        """
+        service = dbus_to_python(service, str)
+        settings = dbus_to_python(settings)
+        log.debug1("config.addService2('%s')", service)
+        self.accessCheck(sender)
+        obj = self.config.new_service_dict(service, settings)
+        config_service = self._addService(obj)
+        return config_service
+
     @dbus.service.signal(config.dbus.DBUS_INTERFACE_CONFIG, signature='s')
     @dbus_handle_exceptions
     def ServiceAdded(self, service):
@@ -1380,7 +1436,9 @@ class FirewallDConfig(slip.dbus.service.Object):
                    (ipv, table, chain, ))
         self.accessCheck(sender)
         settings = list(self.getSettings())
-        settings[1] = []
+        for rule in settings[1][:]:
+            if (ipv, table, chain) == (rule[0], rule[1], rule[2]):
+                settings[1].remove(rule)
         self.update(tuple(settings))
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
